@@ -80,6 +80,8 @@ type verify_error =
 
 type bigarray = (char, Bigarray.int8_unsigned_elt, Bigarray.c_layout) Bigarray.Array1.t
 
+external get_error_string : unit -> string = "ocaml_ssl_get_error_string"
+
 exception Method_error
 exception Context_error
 exception Certificate_error
@@ -96,6 +98,25 @@ exception Read_error of ssl_error
 exception Write_error of ssl_error
 exception Verify_error of verify_error
 
+let () =
+  Printexc.register_printer (function
+    | Method_error -> Some ("SSL: Method error")
+    | Context_error -> Some ("SSL: Context error")
+    | Certificate_error -> Some ("SSL: Certificate error")
+    | Cipher_error -> Some ("SSL: Cihper error")
+    | Diffie_hellman_error -> Some ("SSL: Diffie Hellman error")
+    | Ec_curve_error -> Some ("SSL: EC curve error")
+    | Private_key_error -> Some ("SSL: Privte key error")
+    | Unmatching_keys -> Some ("SSL: Unmatching keys")
+    | Invalid_socket -> Some ("SSL: Invalid socket")
+    | Handler_error -> Some ("SSL: Handler error")
+    | Connection_error _ -> Some ("SSL connection() error: " ^ (get_error_string()))
+    | Accept_error _ -> Some ("SSL accept() error: " ^ (get_error_string()))
+    | Read_error _ -> Some ("SSL read() error: " ^ (get_error_string()))
+    | Write_error _ -> Some ("SSL write() error: " ^ (get_error_string()))
+    | Verify_error _ -> Some ("SSL verify() error: " ^ (get_error_string()))
+    | _ -> None)
+
 let () =
   Callback.register_exception "ssl_exn_method_error" Method_error;
   Callback.register_exception "ssl_exn_context_error" Context_error;
@@ -117,8 +138,6 @@ let thread_safe = ref false
 
 external init : bool -> unit = "ocaml_ssl_init"
 
-external get_error_string : unit -> string = "ocaml_ssl_get_error_string"
-
 let ts = thread_safe
 let init ?thread_safe () =
   let thread_safe =
@@ -206,6 +225,8 @@ external verify : socket -> unit = "ocaml_ssl_verify"
 
 external write : socket -> Bytes.t -> int -> int -> int = "ocaml_ssl_write"
 
+external write_substring : socket -> string -> int -> int -> int = "ocaml_ssl_write"
+
 external write_bigarray : socket -> bigarray -> int -> int -> int = "ocaml_ssl_write_bigarray"
 
 external write_bigarray_blocking :
@@ -246,12 +267,11 @@ let open_connection ssl_method sockaddr =
 let shutdown_connection = shutdown
 
 let output_string ssl s =
-  ignore (write ssl s 0 (String.length s))
+  ignore (write_substring ssl s 0 (String.length s))
 
 let output_char ssl c =
-  let tmp = Bytes.create 1 in
-    Bytes.set tmp 0 c;
-    ignore (write ssl tmp 0 1)
+  let tmp = String.make 1 c in
+    ignore (write_substring ssl tmp 0 1)
 
 let output_int ssl i =
   let tmp = Bytes.create 4 in
@@ -269,7 +289,7 @@ let input_string ssl =
     while !r <> 0
     do
       r := read ssl buf 0 bufsize;
-      ret := !ret ^ (Bytes.sub buf 0 !r)
+      ret := !ret ^ (Bytes.sub_string buf 0 !r)
     done;
     !ret
 
